# -*- coding: UTF-8 -*-
import copy
import sys
import wave
sys.path.append('./')
sys.path.append('../')
from ctypes import *
from commFunction import emxArray_real_T,get_data_of_ctypes_,write_ctypes_data_2_file_,get_none_data_of_ctypes_
import  ctypes


# DLL_EXPORT void matchsig_2(const emxArray_real_T *ref, const emxArray_real_T *sig, double
#                 fs, double type,emxArray_real_T *sig_out, double *delay, double *err)


# void matchsig_2(const emxArray_real_T *ref, const emxArray_real_T *sig, double
#                 fs, double type, emxArray_real_T *sig_out, double *delay, double
#                 *err)

# void matchsig_2(const emxArray_real_T *ref, const emxArray_real_T *sig, double
#                 fs, double type, emxArray_real_T *sig_out, double *delay, double
#                 *err)

def match_sig(refFile=None,testFile=None,outFile=None,audioType=0):
    """
    Parameters
    ----------
    refFile
    testFile
    outFile
    audioType  0:speech,1:music

    Returns
    -------

    """

    refstruct, refsamplerate,reflen = get_data_of_ctypes_(refFile)
    teststruct, testsamplerate,testlen = get_data_of_ctypes_(testFile)
    outlen = max(reflen,testlen)

    outStruct = get_none_data_of_ctypes_(outlen)
    if refsamplerate != testsamplerate :
        raise TypeError('Different format of ref and test files!')

    import platform
    mydll = None
    cur_paltform = platform.platform().split('-')[0]
    if cur_paltform == 'Windows':
        mydll = ctypes.windll.LoadLibrary(sys.prefix + '/matchsig.dll')
    if cur_paltform == 'macOS':
        mydll = CDLL(sys.prefix + '/matchsig.dylib')

    mydll.matchsig_2.argtypes = [POINTER(emxArray_real_T), POINTER(emxArray_real_T), POINTER(emxArray_real_T),c_double,c_double,
                                     POINTER(c_double), POINTER(c_double)]
    delay, err = c_double(0.0), c_double(0.0)
    mydll.matchsig_2(byref(refstruct), byref(teststruct), byref(outStruct),c_double(refsamplerate),c_double(audioType),byref(delay), byref(err))
    if err.value > 0.0:
        return None
    else:
        if outFile is not None:
            write_ctypes_data_2_file_(outFile,outStruct,refsamplerate)
        return delay.value




if __name__ == '__main__':
    ref = r'C:\Users\vcloud_avl\Documents\我的POPO\src.wav'
    test = r'C:\Users\vcloud_avl\Documents\我的POPO\test.wav'
    print(match_sig(refFile=ref, testFile=test, outFile='outfile.wav'))

    pass


# -*- coding: UTF-8 -*-
import sys

sys.path.append('./')
sys.path.append('../')
from ctypes import *
from commFunction import emxArray_real_T,get_data_of_ctypes_,write_ctypes_data_2_file_,get_none_data_of_ctypes_
import ctypes,platform


def MATCH_AEC(refFile=None, testFile=None, caliFile=None,outFile=None,targetType=0):
    """
    %         type- input signal type:
    %               0:Chiness
    %               1:English
    %               2:Single Digit
    %               3:Music
    """
    refstruct,refsamplerate,reflen = get_data_of_ctypes_(refFile)
    teststruct,testsamplerate,testlen = get_data_of_ctypes_(testFile)
    calistruct,calisamplerate,_ = get_data_of_ctypes_(caliFile)
    outlen = max(reflen, testlen)
    outStruct = get_none_data_of_ctypes_(outlen)

    if refsamplerate != testsamplerate or testsamplerate!= calisamplerate:
        raise TypeError('Different format of ref and test files!')

    mydll = None
    cur_paltform = platform.platform().split('-')[0]
    if cur_paltform == 'Windows':
        mydll = ctypes.windll.LoadLibrary(sys.prefix + '/matchsig_aec.dll')
    if cur_paltform == 'macOS':
        mydll = CDLL(sys.prefix + '/matchsig_aec.dylib')

    mydll.matchsig_aec.argtypes = [POINTER(emxArray_real_T),POINTER(emxArray_real_T),POINTER(emxArray_real_T),c_double,c_double,POINTER(emxArray_real_T),POINTER(c_double)]
    err,fs_out,type = c_double(0.0),c_double(refsamplerate),c_double(targetType)
    mydll.matchsig_aec(byref(teststruct),byref(refstruct),byref(calistruct),refsamplerate,type,byref(outStruct),byref(err))

    if err.value == 0.0:
        if outFile is not None:
            write_ctypes_data_2_file_(outFile, outStruct,refsamplerate)
        return True
    else:
        return False


if __name__ == '__main__':
    path = r'D:\AudioPublicWork\3a_auto_test_porject\3a_auto_test_porject\08_TestDstFiles\sdk_zego_vivo_y3hf_music_V_shengbo_compare\aec\Speech\TestCase_01_None_None\near_cn'
    ref = path +'\\' +'far_cn.wav'
    test = path +'\\' + 'mixDstFile.wav'
    cali = path +'\\' + 'mixDstFile.wav'
    outFile = r'C:\Users\vcloud_avl\Downloads\Speech\TestCase_01_None_None\near_cn\target.wav'
    delay = MATCH_AEC(refFile=ref,testFile=test,caliFile=cali,outFile=outFile,targetType=0)
    print(delay)
    pass


# -*- coding: utf-8 -*-
# Andrea Castiella Aguirrezabala
# Psychoacoustic Annoyance Zwicker y Fastl "Psychoacoustics: Facts and Models"
import numpy as np

# Cálculo PA
def psychoacousticAnnoyance(L, S, FS, R):
    N5 = calcPercentile(L)
    if N5 == 0:
        N5 = 1e-8
    ws = 0
    if S > 1.75:
        ws = (S-1.75)*0.25*np.log10(N5+10)     
    wfr = (2.18/N5**0.4)*(0.4*FS + 0.6*R)
    PA = N5*(1 + np.sqrt(ws**2+wfr**2))
    return PA


# Cálculo percentil
def calcPercentile(x, N=5):
    xsort = np.sort(x)
    l = len(x)
    pos = np.ceil(0.01*N*l)
    return xsort[int(pos)-1]


import sys
sys.path.append('../')
import os,time
import socket
import paramiko
from stat import S_ISDIR as isdir
import  copy
from ctypes import *
import wave
import numpy as np
import math
import librosa

class emxArray_real_T(Structure):
 _fields_ = [
          ("pdata", POINTER(c_double)),  # c_byte
          ("psize", POINTER(c_int)),  # c_byte
          ("allocSize", c_int),  #  c_byte
          ("NumDimensions", c_int),  # c_byte
          ("canFreeData", c_uint),
]



def get_data_array(filename):
    """

    """
    f = wave.open(filename, "rb")
    # 读取格式信息
    # 一次性返回所有的WAV文件的格式信息，它返回的是一个组元(tuple)：声道数, 量化位数（byte单位）, 采样频率, 采样点数, 压缩类型, 压缩类型的描述。wave模块只支持非压缩的数据，因此可以忽略最后两个信息
    params = f.getparams()
    nchannels, sampwidth, framerate, nframes = params[:4]
    # 读取波形数据
    # 读取声音数据，传递一个参数指定需要读取的长度（以取样点为单位）
    str_data = f.readframes(nframes)
    f.close()
    return np.frombuffer(str_data, dtype=np.int16),framerate,nchannels

def get_data_array_double(filename):
    """

    """
    f = wave.open(filename, "rb")
    # 读取格式信息
    # 一次性返回所有的WAV文件的格式信息，它返回的是一个组元(tuple)：声道数, 量化位数（byte单位）, 采样频率, 采样点数, 压缩类型, 压缩类型的描述。wave模块只支持非压缩的数据，因此可以忽略最后两个信息
    params = f.getparams()
    nchannels, sampwidth, framerate, nframes = params[:4]
    # 读取波形数据
    # 读取声音数据，传递一个参数指定需要读取的长度（以取样点为单位）
    str_data = f.readframes(nframes)
    f.close()
    return np.frombuffer(str_data, dtype=np.int16).astype(np.double),framerate,nchannels



def get_data_of_ctypes_(inWaveFile=None,int2float=False):
    wavf = wave.open(inWaveFile, 'rb')
    refChannel,refsamWidth,refsamplerate,refframeCount = wavf.getnchannels(),wavf.getsampwidth(),wavf.getframerate(),wavf.getnframes()

    if (refChannel,refsamWidth) != (1,2):
        raise TypeError('Different format of ref and test files!')
    pcmdata = wavf.readframes(refframeCount)

    ref = np.frombuffer(pcmdata,dtype=np.int16)

    ref = ref.astype(np.float64)
    if int2float:
        ref = ref/32768
    datastruct = emxArray_real_T()

    datastruct.pdata = (c_double * refframeCount)(*ref)
    datastruct.psize = (c_int * 1)(*[refframeCount])
    wavf.close()
    return  datastruct,refsamplerate,refframeCount

def get_data_of_ctypes_from_datablock(inData=None,datalen=96000,int2float=False):

    ref = np.frombuffer(inData,dtype=np.int16)

    ref = ref.astype(np.float64)
    if int2float:
        ref = ref/32768
    datastruct = emxArray_real_T()
    datastruct.pdata = (c_double * datalen)(*ref)
    datastruct.psize = (c_int * 1)(*[datalen])

    return  datastruct

def get_none_data_of_ctypes_(dataLength=0):

    data =  np.array([0.0 for _ in range(dataLength)])
    data = data.astype(np.float64)

    outStruct = emxArray_real_T()
    #outStruct = create_string_buffer(20)
    outStruct.pdata =  (c_double * dataLength)(*data)
    outStruct.psize = (c_int * 1)(*[dataLength])
    outStruct.allocSize = dataLength
    outStruct.NumDimensions = 1
    outStruct.canFreeData = 1
    return outStruct



def write_ctypes_data_2_file_(outFile,outStruct,refsamplerate):
    outf = wave.open(outFile, 'wb')
    outf.setnchannels(1)
    outf.setsampwidth(2)
    outf.setframerate(refsamplerate)
    # 将wav_data转换为二进制数据写入文件
    outlist = []
    for a in range(outStruct.psize[0]):
        outlist.append(int(outStruct.pdata[a]))
    outarray = np.array(outlist)
    outarray = outarray.astype(np.int16)
    outf.writeframes(bytes(outarray))
    outf.close()
constMosResult = {'delay':'No Result','mos':'-0.0','Speech Level Gain':'','Noise Level Gain':''}

class commondata():
    def __init__(self):
        self.mosResult = copy.deepcopy(constMosResult)
        self.HOST = '10.219.33.45'
        self.machost = '10.219.33.45' #'10.242.167.159'
        self.username = 'netease'
        self.password = 'Netease163'
        self.PORT = 2159
        self.sftpPort = 22
    @staticmethod
    def get_data():
        return {"type": "command",
        "module": "clientA",
        "method": "requestA",
        "samplerate":16000,
        "token": "",
        "job":None,
        "srcFile":'',
        "testFile":'',
        "result":{},
        "err":"No error"}

global_result = commondata()
def log_time():
    time_tup = time.localtime(time.time())
    # format_time = '%Y-%m-%d_%a_%H-%M-%S'
    format_time = '%Y-%m-%d-%H-%M-%S'

    cur_time = time.strftime(format_time, time_tup)
    return cur_time

def getip():
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('8.8.8.8', 80))
        vqtip = s.getsockname()[0]
    finally:
        s.close()

    return vqtip


def exec_shell_command(cmd):
    ssh = paramiko.SSHClient()
    # 把要连接的机器添加到known_hosts文件中
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    # 连接服务器
    ssh.connect(hostname=serverIP, port=port, username=username, password=password, allow_agent=False,
                look_for_keys=False)
    stdin, stdout, stderr = ssh.exec_command(cmd, get_pty=True)
    result = stdout.read()
    ssh.close()
    return result



def sftp_connect(username,password,host,port=22):
    client = None
    sftp = None
    try:
        client = paramiko.Transport((host,port))
    except Exception as error:
        print(error)
    else:
        try:
            client.connect(username=username, password=password)
        except Exception as error:
            print(error)
        else:
            sftp = paramiko.SFTPClient.from_transport(client)
    return client,sftp


def sftp_disconnect(client):
    try:
        client.close()
    except Exception as error:
        print(error)





def _check_local(local):
    if not os.path.exists(local):
        try:
            os.mkdir(local)
        except IOError as err:
            print(err)


def sftp_get(sftp, remote, local):
    # 检查远程文件是否存在
    try:
        result = sftp.stat(remote)
    except IOError as err:
        error = '[ERROR %s] %s: %s' % (err.errno, os.path.basename(os.path.normpath(remote)), err.strerror)
        print(error)
    else:
        # 判断远程文件是否为目录
        if isdir(result.st_mode):
            dirname = os.path.basename(os.path.normpath(remote))
            local = os.path.join(local, dirname)
            _check_local(local)
            for file in sftp.listdir(remote):
                sub_remote = os.path.join(remote, file)
                sub_remote = sub_remote.replace('\\', '/')
                sftp_get(sftp, sub_remote, local)
        else:
            # 拷贝文件
            if os.path.isdir(local):
                local = os.path.join(local, os.path.basename(remote))
            try:
                sftp.get(remote, local)
            except IOError as err:
                print(err)
            else:
                print('[get]', local, '<==', remote)


def sftp_put(sftp, local, remote):
    # 检查路径是否存在
    def _is_exists(path, function):
        path = path.replace('\\', '/')
        try:
            function(path)
        except Exception as error:
            return False
        else:
            return True

    # 拷贝文件
    def _copy(sftp, local, remote):
        # 判断remote是否是目录
        if _is_exists(remote, function=sftp.chdir):
            # 是，获取local路径中的最后一个文件名拼接到remote中
            filename = os.path.basename(os.path.normpath(local))
            remote = os.path.join(remote, filename).replace('\\', '/')
        # 如果local为目录
        if os.path.isdir(local):
            # 在远程创建相应的目录
            _is_exists(remote, function=sftp.mkdir)
            # 遍历local
            for file in os.listdir(local):
                # 取得file的全路径
                localfile = os.path.join(local, file).replace('\\', '/')
                # 深度递归_copy()
                _copy(sftp=sftp, local=localfile, remote=remote)
        # 如果local为文件
        if os.path.isfile(local):
            try:
                sftp.put(local, remote)
            except Exception as error:
                print(error)
                print('[put]', local, '==>', remote, 'FAILED')
            else:
                print('[put]', local, '==>', remote, 'SUCCESSED')

    # 检查local
    if not _is_exists(local, function=os.stat):
        print("'" + local + "': No such file or directory in local")
        return False
    # 检查remote的父目录
    remote_parent = os.path.dirname(os.path.normpath(remote))
    if not _is_exists(remote_parent, function=sftp.chdir):
        print("'" + remote + "': No such file or directory in remote")
        return False
    # 拷贝文件
    _copy(sftp=sftp, local=local, remote=remote)


def get_file_path(root_path,file_list,dir_list):
    #获取该目录下所有的文件名称和目录名称
    dir_or_files = os.listdir(root_path)
    for dir_file in dir_or_files:
        #获取目录或者文件的路径
        dir_file_path = os.path.join(root_path,dir_file)
        #判断该路径为文件还是路径
        if os.path.isdir(dir_file_path):
            dir_list.append(dir_file_path)
            #递归获取所有文件和目录的路径
            get_file_path(dir_file_path,file_list,dir_list)
        else:
            file_list.append(dir_file_path)


def project_root_path(project_name=None):

    """

    获取当前项目根路径

    :param project_name:

    :return: 根路径

    """

    PROJECT_NAME = 'audiotestalgorithm' if project_name is None else project_name

    project_path = os.path.abspath(os.path.dirname(__file__))

    root_path = project_path[:project_path.find("{}\\".format(PROJECT_NAME)) + len("{}\\".format(PROJECT_NAME))]

    #print('当前项目名称：{}\r\n当前项目根路径：{}'.format(PROJECT_NAME, root_path))

    return root_path


def make_out_file(tarFile,data,fs,channel):
    """

    """
    outData = data.astype(np.int16)
    wavfile = wave.open(tarFile, 'wb')
    wavfile.setnchannels(channel)
    wavfile.setsampwidth(2)
    wavfile.setframerate(fs)
    wavfile.writeframes(outData.tobytes())
    wavfile.close()

def get_ave_rms(data):
    '''
    Parameters
    ----------
    data

    Returns
    -------
    '''
    frameLen = 480
    nFrames = len(data)//frameLen
    totalRms,cnt = 0,0
    for a in range(nFrames):
        totalRms += get_rms(data[a*frameLen:(a+1)*frameLen])
        cnt += 1
    return totalRms/cnt

def get_rms(records):
    '''
    Parameters
    ----------
    records

    Returns
    -------
    '''
    #return math.sqrt(sum([x * x for x in records])/len(records))
    data = records.astype(np.float32).tolist()
    rms = math.sqrt(sum([(x/32767) * (x/32767) for x in data])/len(data))
    dBrmsValue = 20*math.log10(rms + 1.0E-6)
    return dBrmsValue

def get_one_channel_data(infile):
    """
    :return:
    """
    data, fs, chn = get_data_array(infile)
    if fs == 48000 and chn == 1:
        return data
    if chn != 1:
        data = np.array([data[n] for n in range(len(data)) if n % chn == 0]).astype(np.int16)
    return resample(data,fs,48000)

def resample(data,fs,tarfs):
    """
    :return:
    """
    if fs == tarfs:
        return data
    new_signal = librosa.resample(data.astype(np.float32), fs, tarfs)
    # augmenter = Resample(min_sample_rate=4000, max_sample_rate=48000, p=1.0)
    # samples = augmenter(samples=data.astype(np.float32), sample_rate=tarfs)
    return new_signal.astype(np.int16)

def get_file_duration(filename):
    """
    :param filename:
    :return:
    """
    f = wave.open(filename, "rb")
    # 读取格式信息
    # 一次性返回所有的WAV文件的格式信息，它返回的是一个组元(tuple)：声道数, 量化位数（byte单位）, 采样频率, 采样点数, 压缩类型, 压缩类型的描述。wave模块只支持非压缩的数据，因此可以忽略最后两个信息
    params = f.getparams()
    nchannels, sampwidth, framerate, nframes = params[:4]
    # 读取波形数据
    # 读取声音数据，传递一个参数指定需要读取的长度（以取样点为单位）
    f.close()
    return nframes/framerate/nchannels,framerate


# -*- coding:utf-8 -*-
import sys
import  os,time
from os import path
import copy
sys.path.append(os.path.dirname(path.dirname(__file__)))
from commFunction import global_result,sftp_connect,sftp_get,sftp_disconnect,constMosResult
import shutil
from  socketClient import SocketClient
from POLQA import startvqt


def exec_polqa_test():

    while(True):
        try:
            socket = SocketClient(global_result.machost,global_result.PORT)
            curdata = global_result.get_data()
            curdata['module'] = 'clientB'
            curdata['method'] = 'requestB'
            curruslt = socket.sender(curdata)
        except:
            socket.close()
            continue
        if curruslt['job'] is None or str(curruslt['job']) == 'null':
            continue

        #try:
        print('processing')
        #检查文件

        #链接sftp
        client,sftp = sftp_connect(global_result.username,global_result.password,global_result.HOST,port=global_result.sftpPort)
        sftp_get(sftp, '/home/netease/polqa/' + curruslt['token'], '')
        sftp_disconnect(client)
        srf ,tsf, fpath,sr = curruslt['token'] +'/'+ curruslt['srcFile'],curruslt['token'] +'/'+ curruslt['testFile'],curruslt['token'],curruslt['samplerate']
        if not os.path.exists(srf) or not os.path.exists(tsf):
            curruslt['result'] = 'lack of input files!'
            socket = SocketClient(global_result.machost, global_result.PORT)
            curruslt['module'] = 'clientB'
            curruslt['method'] = 'response'
            try:
                douresult = socket.sender(curruslt)
            except:
                socket.close()
                shutil.rmtree(fpath)
                print(fpath,' is not exist')
            continue

        # samplerate = info[3]
        global_result.mosResult = copy.deepcopy(constMosResult)
        startvqt(os.path.abspath(srf), os.path.abspath(tsf), sr)
        if '-0.0' == global_result.mosResult['mos']:
            time.sleep(5)
        curruslt['result'] = global_result.mosResult

        socket = SocketClient(global_result.machost, global_result.PORT)
        curruslt['module'] = 'clientB'
        curruslt['method'] = 'response'
        print(curruslt)
        try:
            socket.sender(curruslt)
        except:
            socket.close()
            time.sleep(1)
            shutil.rmtree(fpath)
            print('file was not deleted!')



if __name__ == '__main__':
    exec_polqa_test()  #0 从头开始  #-1 从尾开始


# -*- coding: UTF-8 -*-
import copy
import sys
import time

sys.path.append('./')
sys.path.append('../')
from ctypes import *
from commFunction import emxArray_real_T,get_data_of_ctypes_from_datablock,get_data_array,get_none_data_of_ctypes_
import  ctypes
import numpy as np

# DLL_EXPORT void matchsig_2(const emxArray_real_T *ref, const emxArray_real_T *sig, double
#                 fs, double type,emxArray_real_T *sig_out, double *delay, double *err)


# void matchsig_2(const emxArray_real_T *ref, const emxArray_real_T *sig, double
#                 fs, double type, emxArray_real_T *sig_out, double *delay, double
#                 *err)

# void matchsig_2(const emxArray_real_T *ref, const emxArray_real_T *sig, double
#                 fs, double type, emxArray_real_T *sig_out, double *delay, double
#                 *err)

def match_sig(refData=None,testData=None,refsamplerate=48000):
    """
    Parameters
    ----------
    refFile
    testFile
    outFile
    audioType  0:speech,1:music

    Returns
    -------

    """
    assert len(refData) == len(testData)
    nframes = len(refData)
    print(time.time())
    refstruct = get_data_of_ctypes_from_datablock(refData,nframes)
    print(time.time())
    teststruct = get_data_of_ctypes_from_datablock(testData,nframes)
    print(time.time())
    outStruct = get_none_data_of_ctypes_(nframes)

    import platform
    mydll = None
    cur_paltform = platform.platform().split('-')[0]
    if cur_paltform == 'Windows':
        mydll = ctypes.windll.LoadLibrary(sys.prefix + '/matchsig.dll')
    if cur_paltform == 'macOS':
        mydll = CDLL(sys.prefix + '/matchsig.dylib')

    mydll.matchsig_2.argtypes = [POINTER(emxArray_real_T), POINTER(emxArray_real_T), POINTER(emxArray_real_T),c_double,c_double,
                                     POINTER(c_double), POINTER(c_double)]
    delay, err = c_double(0.0), c_double(0.0)
    print(time.time())
    mydll.matchsig_2(byref(refstruct), byref(teststruct), byref(outStruct),c_double(refsamplerate),c_double(0),byref(delay), byref(err))
    print(time.time())
    if err.value > 0.0:
        return None
    else:
        return delay.value




if __name__ == '__main__':
    ref = r'C:\Users\vcloud_avl\Documents\我的POPO\src.wav'
    test = r'C:\Users\vcloud_avl\Documents\我的POPO\test.wav'
    refdata,fs,ch = get_data_array(ref)
    testdata,fs,ch = get_data_array(test)
    testdata = testdata[:96000]
    refdata = refdata[:96000]
    print(time.time())
    print(match_sig(refData=refdata, testData=testdata))
    print(time.time())
    print(np.corrcoef(refdata,testdata))
    print(time.time())
    pass


#!/usr/bin/python

"""
Speech Transmission Index (STI) test script

Copyright (C) 2011 Jon Polom <jmpolom@wayne.edu>
Licensed under the GNU General Public License
"""

from datetime import date
from .sti import stiFromAudio, readwav
import  numpy as np

__author__ = "Jonathan Polom <jmpolom@wayne.edu>"
__date__ = date(2011, 4, 22)
__version__ = "0.5"

def cal_sti(refFile,testFile):
    # read audio
    refAudio, refRate = readwav(refFile)
    degrAudio, degrRate = readwav(testFile)

    ntimes = 20 // (len(refAudio) // refRate) + 2
    finalrefAudio, finaldegrAudio = np.repeat(refAudio, ntimes, 0), np.repeat(degrAudio, ntimes, 0)
    # calculate the STI. Visually verify console output.
    stis = stiFromAudio(finalrefAudio, finaldegrAudio, refRate, name=testFile)
    
    return stis

if __name__ == '__main__':
    cal_sti('','')



# -*- coding: UTF-8 -*-
from setuptools import setup
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='AlgorithmLib',
    version='3.3.5',
    packages=setuptools.find_packages(),
    url='https://github.com/pypa/sampleproject',
    license='MIT',
    author=' MA JIANLI',
    author_email='majianli@corp.netease.com',
    description='audio algorithms to compute and test audio quality of speech enhencement',
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=[
    'numpy',
    'wave',
    'matplotlib',
    'datetime',
    'scipy',
    'pystoi',
    'paramiko',
    'moviepy',
    'torch',
    'librosa',
    'requests',
    'pandas',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    data_files=[
                ('', ['algorithmLib/DLLs/p563.dll']),
                ('', ['algorithmLib/DLLs/p563.dylib']),
                ('', ['algorithmLib/DLLs/resampler.dll']),
                ('', ['algorithmLib/DLLs/resample.dylib']),
                ('', ['algorithmLib/DLLs/g160.dll']),
                ('', ['algorithmLib/DLLs/g160.dylib']),
                ('', ['algorithmLib/DLLs/cygwin1.dll']),
                ('', ['algorithmLib/DLLs/peaqb.exe']),
                ('', ['algorithmLib/DLLs/PY_PESQ.dll']),
                ('', ['algorithmLib/DLLs/PY_PESQ.dylib']),
                ('', ['algorithmLib/DLLs/matchsig.dll']),
                ('', ['algorithmLib/DLLs/matchsig.dylib']),
                ('', ['algorithmLib/DLLs/snr_music.dll']),
                ('', ['algorithmLib/DLLs/snr_music.dylib']),
                ('', ['algorithmLib/DLLs/snr_transient.dll']),
                ('', ['algorithmLib/DLLs/snr_transient.dylib']),
                ('', ['algorithmLib/DLLs/time_align.dll']),
                ('', ['algorithmLib/DLLs/time_align.dylib']),
                ('', ['algorithmLib/DLLs/agcDelay.dll']),
                ('', ['algorithmLib/DLLs/agcDelay.dylib']),
                ('', ['algorithmLib/DLLs/attackrelease.dll']),
                ('', ['algorithmLib/DLLs/attackrelease.dylib']),
                ('', ['algorithmLib/DLLs/gaintable.dll']),
                ('', ['algorithmLib/DLLs/gaintable.dylib']),
                ('', ['algorithmLib/DLLs/musicStability.dll']),
                ('', ['algorithmLib/DLLs/musicStability.dylib']),
                ('', ['algorithmLib/DLLs/matchsig_aec.dll']),
                ('', ['algorithmLib/DLLs/matchsig_aec.dylib']),
                ('', ['algorithmLib/DLLs/ERLE_estimation.dll']),
                ('', ['algorithmLib/DLLs/ERLE_estimation.dylib']),
                ('', ['algorithmLib/DLLs/pcc.dll']),
                ('', ['algorithmLib/DLLs/pcc.dylib']),
                ('', ['algorithmLib/DLLS/SC_res_retrain_220316_185754125621__ep_007.tar']),
                ],

    python_requires='>=3.7',
)






# -*- coding: UTF-8 -*-
import sys

sys.path.append('../')
from ctypes import *
from commFunction import emxArray_real_T,get_data_of_ctypes_
import ctypes

# void gaintable_estimation(const emxArray_real_T *ref, const emxArray_real_T *sig,
#   double fs_ref, double fs_sig, double type, double gain_table[11], double DR[11],
#   double *limiter, double *err)

def cal_gain_table(refFile=None, testFile=None,targetType=0):
    '''
    Parameters
    ----------
    refFile
    testFile
    targetType  0:speech,1:music

    Returns
    -------

    '''
    refstruct,refsamplerate,_ = get_data_of_ctypes_(refFile,True)
    teststruct,testsamplerate,_ = get_data_of_ctypes_(testFile,True)

    if refsamplerate != testsamplerate :
        raise TypeError('Different format of ref and test files!')

    import platform
    mydll = None
    cur_paltform = platform.platform().split('-')[0]
    if cur_paltform == 'Windows':
        mydll = ctypes.windll.LoadLibrary(sys.prefix + '/gaintable.dll')
    if cur_paltform == 'macOS':
        mydll = CDLL(sys.prefix + '/gaintable.dylib')

    mydll.gaintable_estimation.argtypes = [POINTER(emxArray_real_T),POINTER(emxArray_real_T),c_double,c_double,c_double,POINTER(c_double),POINTER(c_double),POINTER(c_double),POINTER(c_double)]
    data_format = c_double*11
    gain_table = data_format()
    DR = data_format()
    limiter,err = c_double(0.0),c_double(0.0)
    mydll.gaintable_estimation(byref(refstruct),byref(teststruct),c_double(refsamplerate),c_double(refsamplerate),c_double(targetType),gain_table,DR,byref(limiter),byref(err))

    if err.value == 0.0:
        return limiter.value,gain_table,DR
    else:
        return None


if __name__ == '__main__':
    file = r'C:\Users\vcloud_avl\Documents\我的POPO\0\speech_gaintable.wav'
    test = r'C:\Users\vcloud_avl\Documents\我的POPO\0\speech_gaintable.wav'
    lim,gain_table,DR = cal_gain_table(refFile=file,testFile=test,targetType=0)
    print(lim,gain_table[0],DR[2])
    print(gain_table,DR)
    for a in gain_table:
        print(a)
    for a in DR:
        print(a)
    pass



import sys
import  os
import time
from os import  path
sys.path.append(os.path.dirname(path.dirname(__file__)))
from formatConvert.wav_pcm import wav2pcm,pcm2wav
from G160.G160 import cal_g160
from P563.P563 import cal_563_mos
from PESQ.PESQ import cal_pesq
from POLQA.polqa_client import  polqa_client_test
from SDR.SDR import cal_sdr
from STI.cal_sti import cal_sti
from STOI.STOI import cal_stoi
from PEAQ.PEAQ import cal_peaq
from resample.resampler import resample,restruct
from timeAligment.time_align import cal_delay
import os
import wave
import numpy as np
from ctypes import  *
from SNR_ESTIMATION.MATCH_SIG import match_sig
from SNR_ESTIMATION.SNR_MUSIC import cal_snr_music
from SNR_ESTIMATION.SNR_TRANSIENT import cal_snr_transient
from AGC_EVALUATION.CAL_GAIN_TABLE import cal_gain_table
from AGC_EVALUATION.CAL_ATTACK_RELEASE import cal_attack_release
from AGC_EVALUATION.CAL_MUSIC_STABILITY import cal_music_stablility
from AGC_EVALUATION.CAL_DELAY import cal_DELAY
from AEC_EVALUATION.MATCH_AEC import MATCH_AEC
from AEC_EVALUATION.ERLE_ETSIMATION import cal_erle
from AEC_MOS.aecmos import cal_aec_mos
from MOS_INFER.run_predict import cal_mos_infer
from FUNCTION.audioFunction import isSlience,audioFormat,get_rms_level
from Noise_Suppression.noiseFuction import  cal_noise_Supp

allMetrics = ['G160','P563','POLQA','PESQ','STOI','STI','PEAQ','SDR',
              'SII','LOUDNESS','MUSIC','TRANSIENT','MATCH','GAINTABLE',
              'ATTACKRELEASE','MUSICSTA','AGCDELAY','SLIENCE','FORMAT',
              'MATCHAEC','ERLE','AECMOS','AIMOS','TRMS','ARMS','NOISE']


class computeAudioQuality():
    def __init__(self,**kwargs):
        """
        :param kwargs:
        """
        #print(**kwargs)
        self.__parse_para(**kwargs)
        self.__chcek_valid()
        pass

    def __parse_para(self,**kwargs):
        """
        :param kwargs:
        :return:
        """
        self.mertic = kwargs['metrics']
        self.testFile = kwargs['testFile']
        self.refFile = kwargs['refFile']
        self.micFile = kwargs['micFile']
        self.cleFile = kwargs['cleFile']
        self.noiseFile = kwargs['noiseFile']
        self.caliFile = kwargs['aecCaliFile']
        self.outFile = kwargs['outFile']
        self.samplerate = kwargs['samplerate']
        self.bitwidth = kwargs['bitwidth']
        self.channel = kwargs['channel']
        self.refOffset = kwargs['refOffset']
        self.testOffset = kwargs['refOffset']
        self.maxComNLevel = kwargs['maxComNLevel']
        self.speechPauseLevel = kwargs['speechPauseLevel']
        self.audioType = kwargs["audioType"]
        self.aecStartPoint = kwargs['aecStartPoint']
        self.aecScenario = kwargs['aecScenario']
        self.aecTargetType = kwargs["aecTargetType"]
        self.rmsCalsection = kwargs["rmsCalsection"]
        #maxComNLevel=c_double(-48.0),speechPauseLevel=c_double(-35.0)

    def __chcek_valid(self):
        """
        :return:
        """
        if self.mertic not in allMetrics:
            raise ValueError('matrix must betwin ' + str(allMetrics))

    def __check_format(self,curWav):
        """
        :param curWav:
        :return:
        """
        curType = os.path.splitext(curWav)[-1]
        if curType !='.wav':
            return self.channel,self.bitwidth,self.samplerate
        wavf = wave.open(curWav,'rb')
        curChannel = wavf.getnchannels()
        cursamWidth = wavf.getsampwidth()
        cursamplerate = wavf.getframerate()
        wavf.close()
        if curChannel != 1:
            raise ValueError('wrong type of channel' + curWav)
        if cursamWidth != 2:
            raise ValueError('wrong type of samWidth' + curWav)
        return curChannel,cursamWidth,cursamplerate

    def __double_end_check(self):
        """
        :return:
        """
        if  self.refFile is None or self.testFile is None:
            raise EOFError('lack of inputfiles!')
        if self.__check_format(self.testFile) != self.__check_format(self.refFile):
            raise TypeError('there are different parametre in inputfiles!')

    def __data_convert(self,ref,test):
        """
        :return:
        """
        with open(wav2pcm(ref), 'rb') as ref:
            pcmdata = ref.read()
        with open(wav2pcm(test), 'rb') as ref:
            indata = ref.read()
        ref = np.frombuffer(pcmdata, dtype=np.int16)
        ins = np.frombuffer(indata, dtype=np.int16)
        lenth = min(len(ref),len(ins))
        return ref[:lenth],ins[:lenth]

    def G160(self):
        """
        :return:
        # g160 无采样率限制
        # WAV/PCM 输入
        """
        if self.cleFile is None or self.refFile is None or self.testFile is None:
            raise EOFError('lack of inputfiles!')
        if self.__check_format(self.testFile) != self.__check_format(self.refFile) or \
            self.__check_format(self.testFile) != self.__check_format(self.cleFile):
            raise TypeError('there are different parametre in inputfiles!')
        return cal_g160(pcm2wav(self.cleFile,sample_rate=self.samplerate),pcm2wav(self.refFile,sample_rate=self.samplerate),pcm2wav(self.testFile,sample_rate=self.samplerate),self.refOffset,self.testOffset,maxComNLevel=self.maxComNLevel,speechPauseLevel=self.speechPauseLevel)

    def P563(self):
        """
        # P 563 PCM输入 、 8Khz
        # • Sampling frequency: 8000 Hz
        #  If higher frequencies are used for recording, a separate down-sampling by using a high
        # quality flat low pass filter has to be applied. Lower sampling frequencies are not allowed.
        # • Amplitude resolution: 16 bit linear PCM
        # • Minimum active speech in file: 3.0 s
        # • Maximum signal length: 20.0 s
        # • Minimum speech activity ratio: 25%
        # • Maximum speech activity ratio: 75%
        :return:
        """
        if self.testFile is None:
            raise EOFError('lack of inputfiles!')
        curCH,curBwidth,curSR = self.__check_format(self.testFile)
        #TODO 将采样率
        if curSR != 8000:
            print('file will be resampled to 8k!')
        finalName = wav2pcm(resample(pcm2wav(self.testFile,sample_rate=self.samplerate),8000))
        return cal_563_mos(finalName)

    def POLQA(self):
        """
        #POLQA  窄带模式  8k   超宽带模式 48k
        # pcm输入
        :return:
        """

        self.__double_end_check()
        curCH,curBwidth,curSR = self.__check_format(self.testFile)

        result =  polqa_client_test(wav2pcm(self.refFile),wav2pcm(self.testFile),curSR)
        time.sleep(2)
        return  result


    def PESQ(self):
        """
        # PESQ 窄带模式8K  宽带模式 16k
        # 数据块输入
        :return:
        """
        self.__double_end_check()
        curCH,curBwidth,curSR = self.__check_format(self.testFile)
        if curSR < 16000:
            print('file will be resampled to 8k!')
            finalrefName = wav2pcm(resample(pcm2wav(self.refFile, curSR), 8000))
            finaltestName = wav2pcm(resample(pcm2wav(self.testFile, curSR), 8000))
            return cal_pesq(finalrefName, finaltestName, 8000)
        else:
            print('file will be resampled to 16k!')
            finalrefName = wav2pcm(resample(pcm2wav(self.refFile, sample_rate=curSR), 16000))
            finaltestName = wav2pcm(resample(pcm2wav(self.testFile, sample_rate=curSR), 16000))
            return cal_pesq(finalrefName,finaltestName,16000)


    def STOI(self):
        """
        #STOI
        #数据块输入
        #采样率 16000
        :return:
        """
        self.__double_end_check()
        ref, ins = self.__data_convert(wav2pcm(resample(pcm2wav(self.refFile),16000)),wav2pcm(resample(pcm2wav(self.testFile),16000)))
        result = cal_stoi(ref,ins,sr=16000)
        return result
        pass

    def STI(self):
        """
        #sti
        #wav输入 采样率无关
        :return:
        """
        self.__double_end_check()
        return cal_sti(pcm2wav(self.refFile,sample_rate=self.samplerate),pcm2wav(self.testFile,sample_rate=self.samplerate))
        pass

    def SII(self):
        """
        Returns
        -------

        """
        pass

    def PEAQ(self):
        """
        # wav输入
        :return:
        """
        self.__double_end_check()
        curCH,curBwidth,curSR = self.__check_format(self.testFile)
        if curSR not in [8000,16000]:
            #TODO 采样率
            pass
        #TODO 计算peaq
        return cal_peaq(pcm2wav(self.refFile,sample_rate=self.samplerate),pcm2wav(self.testFile,sample_rate=self.samplerate))
        pass

    def SDR(self):
        """
        #SDR
        #数据块输入  采样率无关
        :return:
        """
        self.__double_end_check()
        ref, ins = self.__data_convert(wav2pcm(resample(pcm2wav(self.refFile),16000)),wav2pcm(resample(pcm2wav(self.testFile),16000)))
        result = cal_sdr(ref,ins,sr=16000)
        return result
        pass

    def MUSIC(self):
        """
        # MUSIC SNR
        # 无采样率限制
        # WAV/PCM 输入
        :return:
        """
        self.__double_end_check()
        return cal_snr_music(refFile=pcm2wav(self.refFile,sample_rate=self.samplerate),testFile=pcm2wav(self.testFile,sample_rate=self.samplerate))

    def TRANSIENT(self):
        """
        # Transient noise SNR
        # 无采样率限制
        # WAV/PCM 输入
        :return:
        """
        if self.cleFile is None or self.testFile is None or self.noiseFile is None:
            raise EOFError('lack of inputfiles!')
        if self.__check_format(self.cleFile) != self.__check_format(self.testFile) or \
            self.__check_format(self.testFile) != self.__check_format(self.noiseFile):
            raise TypeError('there are different parametre in inputfiles!')
        return cal_snr_transient(pcm2wav(self.cleFile,sample_rate=self.samplerate),pcm2wav(self.noiseFile,sample_rate=self.samplerate),pcm2wav(self.testFile,sample_rate=self.samplerate))

    def MATCH(self):
        """
        # MATCH SIG
        # 无采样率限制
        # 可选择是否输出文件
        # WAV/PCM 输入
        :return:
        """
        self.__double_end_check()
        return match_sig(pcm2wav(self.refFile,sample_rate=self.samplerate), pcm2wav(self.testFile,sample_rate=self.samplerate), self.outFile,self.audioType)

    def LOUDNESS(self):
        """
        Returns
        -------

        """
        pass

    def __cal_sii__(self):
        '''
        Returns
        -------

        '''
        #return cal_sii()
        pass

    def GAINTABLE(self):
        """
        AGC PARA 1
        计算agc的gain table
        :return:
        """
        self.__double_end_check()
        return cal_gain_table(refFile=pcm2wav(self.refFile, sample_rate=self.samplerate),
                             testFile=pcm2wav(self.testFile, sample_rate=self.samplerate),targetType=self.audioType)

    def ATTACKRELEASE(self):
        """
        AGC PARA 2
        计算agc的attack release
        :return:
        """
        self.__double_end_check()
        return cal_attack_release(refFile=pcm2wav(self.refFile, sample_rate=self.samplerate),
                             testFile=pcm2wav(self.testFile, sample_rate=self.samplerate))
    def MUSICSTA(self):
        """
        AGC PARA 3
        计算music 信号稳定性
        :return:
        """
        self.__double_end_check()
        return cal_music_stablility(refFile=pcm2wav(self.refFile, sample_rate=self.samplerate),
                             testFile=pcm2wav(self.testFile, sample_rate=self.samplerate))

    def AGCDELAY(self):
        """
        AGC PARA 3
        计算文件延时
        :return:
        """
        self.__double_end_check()
        return cal_DELAY(refFile=pcm2wav(self.refFile, sample_rate=self.samplerate),
                             testFile=pcm2wav(self.testFile, sample_rate=self.samplerate))

    def AECMOS(self):
        """
        Returns
        -------

        """
        if self.refFile is None or self.micFile is None or self.testFile is None:
            raise EOFError('lack of inputfiles!')
        if self.__check_format(self.refFile) != self.__check_format(self.micFile) or \
            self.__check_format(self.micFile) != self.__check_format(self.testFile):
            raise TypeError('there are different parametre in inputfiles!')
        return cal_aec_mos(pcm2wav(self.refFile,sample_rate=self.samplerate),pcm2wav(self.micFile,sample_rate=self.samplerate),pcm2wav(self.testFile,sample_rate=self.samplerate),scenario=self.aecScenario,startPoint=self.aecStartPoint,SAMPLE_RATE=self.samplerate)

    def AIMOS(self):
        """
        Returns
        -------

        """
        if self.testFile is None:
            raise EOFError('lack of inputfiles!')
        finalName = pcm2wav(self.testFile,sample_rate=self.samplerate)
        return cal_mos_infer(finalName)

    def ERLE(self):
        """
        Returns
        -------

        """
        if self.refFile is None or self.micFile is None or self.testFile is None:
            raise EOFError('lack of inputfiles!')
        if self.__check_format(self.refFile) != self.__check_format(self.micFile) or \
            self.__check_format(self.micFile) != self.__check_format(self.testFile):
            raise TypeError('there are different parametre in inputfiles!')
        return cal_erle(refFile=pcm2wav(self.refFile,sample_rate=self.samplerate),micFile=pcm2wav(self.micFile,sample_rate=self.samplerate),testFile=pcm2wav(self.testFile,sample_rate=self.samplerate),targetType=self.aecTargetType)

    def MATCHAEC(self):
        """
        Returns
        -------

        """
        if self.caliFile is None or self.refFile is None or self.testFile is None:
            raise EOFError('lack of inputfiles!')
        if self.__check_format(self.caliFile) != self.__check_format(self.refFile) or \
            self.__check_format(self.caliFile) != self.__check_format(self.testFile):
            raise TypeError('there are different parametre in inputfiles!')
        return MATCH_AEC(pcm2wav(self.refFile,sample_rate=self.samplerate),pcm2wav(self.testFile,sample_rate=self.samplerate),pcm2wav(self.caliFile,sample_rate=self.samplerate),self.outFile,targetType=self.aecTargetType)

    def SLIENCE(self):
        """
        Returns
        -------

        """
        if self.testFile is None:
            raise EOFError('lack of inputfiles!')
        return isSlience(self.testFile,sample_rate=self.samplerate,bits=self.bitwidth,channels=self.channel,section=self.rmsCalsection)

    def FORMAT(self):
        """
        Returns
        -------

        """
        if self.testFile is None:
            raise EOFError('lack of inputfiles!')
        return audioFormat(self.testFile)


    def TRMS(self):
        """
        Returns
        -------
        # (wavFileName=None,rmsMode='total',startTime=0,endTime=1):
        """
        if self.testFile is None:
            raise EOFError('lack of inputfiles!')
        return get_rms_level(wavFileName=pcm2wav(self.testFile,sample_rate=self.samplerate),rmsMode='total',section=self.rmsCalsection)
    def ARMS(self):
        """
        Returns
        -------

        """
        if self.testFile is None:
            raise EOFError('lack of inputfiles!')
        return get_rms_level(wavFileName=pcm2wav(self.testFile,sample_rate=self.samplerate),rmsMode='average',section=self.rmsCalsection)

    def NOISE(self):
        """
        Returns
        -------

        """
        self.__double_end_check()
        return cal_noise_Supp(pcm2wav(self.refFile, sample_rate=self.samplerate),
                             pcm2wav(self.testFile, sample_rate=self.samplerate))